@@ -120,22 +120,41 @@ def get_or_make_label_encoder(params, problem: str, mode: str, label_list=None)
     Returns:
         LabelEncoder -- label encoder
     """
-    if label_list is None:
-        return None
+
     problem_path = params.ckpt_dir
     create_path(problem_path)
+    problem_type = params.problem_type[problem]
     le_path = os.path.join(problem_path, '%s_label_encoder.pkl' % problem)
-    is_seq2seq_text = params.problem_type[problem] == 'seq2seq_text'
-    is_multi_cls = params.problem_type[problem] == 'multi_cls'
-    is_seq = params.problem_type[problem] == 'seq_tag'
+    is_seq2seq_text = problem_type == 'seq2seq_text'
+    is_multi_cls = problem_type == 'multi_cls'
+    is_seq = problem_type == 'seq_tag'
+    is_pretrain = problem_type == 'pretrain'
+    is_masklm = problem_type == 'masklm'
+
+    if problem_type not in params.predefined_problem_type:
+        is_custom = True
+    else:
+        is_custom = False
+
+    if is_pretrain:
+        return None
 
     if mode == 'train' and not os.path.exists(le_path):
-
+        if is_custom:
+            get_or_make_custom_le_fn = params.get_or_make_label_encoder_fn_dict[
+                problem_type]
+            get_or_make_custom_le_fn(params, problem, mode, label_list)
         if is_seq2seq_text:
             label_encoder = load_transformer_tokenizer(
                 params.transformer_decoder_tokenizer_name, params.transformer_decoder_tokenizer_loading)
             pickle.dump(label_encoder, open(le_path, 'wb'))
 
+        elif is_masklm:
+            label_encoder = load_transformer_tokenizer(
+                params.transformer_tokenizer_name, params.transformer_tokenizer_loading
+            )
+            pickle.dump(label_encoder, open(le_path, 'wb'))
+
         elif is_multi_cls:
             label_encoder = MultiLabelBinarizer()
             label_encoder.fit(label_list)
@@ -154,14 +173,26 @@ def get_or_make_label_encoder(params, problem: str, mode: str, label_list=None)
             label_encoder.dump(le_path)
 
     else:
+        if is_custom:
+            get_or_make_custom_le_fn = params.get_or_make_label_encoder_fn_dict[
+                problem_type]
+            label_encoder = get_or_make_custom_le_fn(
+                params, problem, mode, label_list)
 
-        if is_seq2seq_text or is_multi_cls:
+        if is_seq2seq_text or is_multi_cls or is_masklm:
             label_encoder = pickle.load(open(le_path, 'rb'))
         else:
             label_encoder = LabelEncoder()
             label_encoder.load(le_path)
 
-    if not is_seq2seq_text:
+    if is_custom:
+        if problem not in params.num_classes:
+            raise ValueError(
+                'Seems custom get or make label encoder fn dose not set num_classes to'
+                'params. Please specify num_classes. Example: params.num_classes[problem] = 100')
+        return label_encoder
+
+    if not is_seq2seq_text and not is_masklm:
         if is_multi_cls:
             params.num_classes[problem] = label_encoder.classes_.shape[0]
         else:
@@ -311,7 +342,11 @@ def load_transformer_model(model_name_or_config, load_module_name=None):
         load_module = transformers.TFAutoModel
 
     if isinstance(model_name_or_config, str):
-        model = load_module.from_pretrained(model_name_or_config)
+        try:
+            model = load_module.from_pretrained(model_name_or_config)
+        except OSError:
+            model = load_module.from_pretrained(
+                model_name_or_config, from_pt=True)
     else:
         model = load_module.from_config(model_name_or_config)
     return model
@@ -336,6 +371,5 @@ def get_transformer_main_model(model, key='embeddings'):
 
 
 def get_embedding_table_from_model(model):
-    model.bert.embeddings.build(1)
     base_model = get_transformer_main_model(model)
     return base_model.embeddings.word_embeddings
